# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
## GOODNESS OF FIT =============================================================

# Description:
#     Calculate the goodness of fit metrics for models of increasing complexity
#     that are most closely related to the final selected model.

# Paper:
#     Compound and cascading effects of climatic extremes on dengue outbreak
#     risk in the Caribbean: an impact-based modelling framework with long-lag
#     and short-lag interactions

# Script authors:
#     Chloe Fletcher        ORCID: 0000-0002-6705-7605
#     Prof. Rachel Lowe     ORCID: 0000-0003-3939-7343

# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

## Source packages, functions and data -----------------------------------------

# load packages and functions
source("functions/00_packages_functions.R")

# read in harmonised data
data <- read.csv("data/barbados_data.csv")


## Run models -----------------------------------------------------------------

# random effects
re1 <- paste("f(month, model = 'rw2', cyclic = TRUE, constr = TRUE,",
             "scale.model = TRUE, hyper = precision.prior)")
re2 <- "f(year_index, model = 'rw2', hyper = precision.prior)"

# set base formula to intercept only
baseformula <- paste(c("cases ~ 1"), collapse = " + ")

# generate models with intercept + covariates
m0.0 <- runinlamod(formula(baseformula), data)
m0.1 <- runinlamod(formula(form.update("tas.3mon.4")), data)
m0.2 <- runinlamod(formula(form.update("spi6.5")), data)
m0.3 <- runinlamod(formula(form.update("spi6.1")), data)
m0.4 <- runinlamod(formula(form.update("logdir.4")), data)
m0.5 <- runinlamod(formula(form.update("tas.3mon.4 + spi6.5 + spi6.1")), data)
m0.6 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1")), data)
m0.7 <- runinlamod(formula(form.update("tas.3mon.4 + spi6.5 + spi6.1 + logdir.4")), data)
m0.8 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1 + logdir.4")), data)
m0.9 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1 + dir.4")), data)

# set base formula to intercept plus seasonal effect
baseformula <- paste(c("cases ~ 1", re1), collapse = " + ")

# generate models with intercept + seasonal + covariates
m1.0 <- runinlamod(formula(baseformula), data)
m1.1 <- runinlamod(formula(form.update("tas.3mon.4")), data)
m1.2 <- runinlamod(formula(form.update("spi6.5")), data)
m1.3 <- runinlamod(formula(form.update("spi6.1")), data)
m1.4 <- runinlamod(formula(form.update("logdir.4")), data)
m1.5 <- runinlamod(formula(form.update("tas.3mon.4 + spi6.5 + spi6.1")), data)
m1.6 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1")), data)
m1.7 <- runinlamod(formula(form.update("tas.3mon.4 + spi6.5 + spi6.1 + logdir.4")), data)
m1.8 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1 + logdir.4")), data)
m1.9 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1 + dir.4")), data)

# set base formula to intercept plus seasonal and interannual effects
baseformula <- paste(c("cases ~ 1", re1, re2), collapse = " + ")

# generate models with intercept + seasonal + covariates
m2.0 <- runinlamod(formula(baseformula), data)
m2.1 <- runinlamod(formula(form.update("tas.3mon.4")), data)
m2.2 <- runinlamod(formula(form.update("spi6.5")), data)
m2.3 <- runinlamod(formula(form.update("spi6.1")), data)
m2.4 <- runinlamod(formula(form.update("logdir.4")), data)
m2.5 <- runinlamod(formula(form.update("tas.3mon.4 + spi6.5 + spi6.1")), data)
m2.6 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1")), data)
m2.7 <- runinlamod(formula(form.update("tas.3mon.4 + spi6.5 + spi6.1 + logdir.4")), data)
m2.8 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1 + logdir.4")), data)
m2.9 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1 + dir.4")), data)

# create empty gof dataframe
name <- c(paste0("m", paste(rep(0:2, each = 10), rep(0:9, 3), sep = ".")))
mods <- mget(name)
effects <- rep(c("intercept",
                 "intercept + seasonal",
                 "intercept + seasonal + interannual"), each = 10)
covars <- rep(c("", "tas.3mon.4", "spi6.5", "spi6.1", "logdir.4",
                "tas.3mon.4 + spi6.5 + spi6.1",
                "tas.3mon.4 * spi6.5 * spi6.1",
                "tas.3mon.4 + spi6.5 + spi6.1 + logdir.4",
                "tas.3mon.4 * spi6.5 * spi6.1 + logdir.4",
                "tas.3mon.4 * spi6.5 * spi6.1 + dir.4"), 3)
vars <- ifelse(covars == "", effects, c(paste(effects, covars, sep = " + ")))
gof <- data.frame(id = name, mod = vars, dic = NA, waic = NA, mae = NA, rsq = NA)

# tabulate gof for all models
for (m in 1:length(name)){
  gof[m, "dic"] <- dic(mods[[name[m]]])
  gof[m, "waic"] <- waic(mods[[name[m]]])
  gof[m, "mae"] <- mae(data$cases, mods[[name[m]]])
  gof[m, "rsq"] <- rsq(mods[[name[m]]], m0.0)
}

# supplementary table 3
gof_st03 <- gof[sort(c(outer(c(1, 6:7, 9), 0:2 * 10, "+"))), ]


## Plot best model -------------------------------------------------------------

# read in spi-6 models and lagged dir model
spi6.mod.out <- readRDS("outputs/spi6_mods.rds")
base.mod <- readRDS("outputs/base_mod.rds")
idx <- nrow(spi6.mod.out$mod.gof) + 1

spi6.mod.out$mod.gof[idx,"vars"] <- "base"
spi6.mod.out$fitted[[idx]] <- base.mod$summary.fitted.values
spi6.mod.out$fixed[[idx]] <- base.mod$summary.fixed
spi6.mod.out$random[[idx]] <- base.mod$summary.random

# plot temporal and yearly random effects
selection <- c(idx, 58)

re_m <- plot.temp.res(mods = spi6.mod.out$random[selection], titles = c("1", "2"))
re_m + scale_colour_manual(values = c("turquoise4", "#FF5C33"),
                           labels = c("Lagged DIR Model", "Interaction Model"))
#ggsave("outputs/figs/SF_05B_1.png", width = 15, height = 12, dpi = 300)

xlabs <- paste0(min(data$cal_year):(max(data$cal_year) - 1), "/",
                 (min(data$cal_year) + 1):max(data$cal_year))
re_y <- plot.year.res(mods = spi6.mod.out$random[selection], titles = c("1", "2"),
                      xlabs = xlabs)
re_y + scale_colour_manual(values = c("turquoise4", "#FF5C33"),  # colours in reverse due to levels
                           labels = c("Lagged DIR Model", "Interaction Model"))
#ggsave("outputs/figs/SF_05B_2.png", width = 15, height = 12, dpi = 300)

# plot fixed effects for best model only
coeffs <- c(expression(beta["  T"]), expression(beta["  L"]), 
            expression(beta["  S"]), expression(eta["   "]),
            expression(beta["  T,L"]), expression(beta["  T,S"]), 
            expression(beta["  L,S"]), expression(beta["  T,L,S"]))

mod_fe <- spi6.mod.out$fixed[[58]]
mod_fe <- mod_fe[c(3:5, 2, 6:9), ]

mod_fe %>% ggplot() +
  geom_pointrange(aes(x = mean, xmin = `0.025quant`, xmax = `0.975quant`,
                      y = factor(1:nrow(mod_fe))), colour = "#AA1155") +
  geom_vline(xintercept=0, linetype="dashed", col="grey") +
  scale_y_discrete(labels=rev(coeffs), limits = rev) +
  scale_x_continuous(minor_breaks=seq(-0.5, 1, 0.1), breaks=seq(-0.5, 1, 0.5),
                     limits=c(-0.6, 1)) +
  labs(x="Effect Size", y="Coefficients") +
  theme_bw() +
  theme(axis.text.x=element_text(size=9, hjust=1),
        axis.title=element_text(size=12)) +
  panel_border(remove = TRUE)
#ggsave("outputs/figs/SF_05C.png", width = 15, height = 7, dpi = 300)

# plot fixed effects for best model fit 1999-2022 and 1999-2012
data12 <- data[1:(nrow(data) - 12 * 10), ]
m2.8.12 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1 + logdir.4")), data12)

mod_fe_12 <- m2.8.12$summary.fixed
mod_fe_12 <- mod_fe_12[2:nrow(mod_fe_12), ]

tt <- rbind(mod_fe, mod_fe_12)
tt$model <- c(rep(1, 8), rep(2, 8))
tt$coeff <- rep(1:8, 2)

tt %>% ggplot() +
  geom_pointrange(aes(x = mean, xmin = `0.025quant`, xmax = `0.975quant`,
                      y = factor(coeff), col = factor(model)),
                  position = position_dodge(0.5)) +
  geom_vline(xintercept=0, linetype="dashed", col="grey50") +
  scale_color_manual(values = c("#369683", "#AA1155"),
                     labels = c("Up to 2011/12", "Up to 2021/22"),
                     name = "", limits = c(2, 1)) +
  scale_y_discrete(labels = rev(coeffs), limits = rev) +
  scale_x_continuous(minor_breaks = seq(-0.7, 1, 0.1), breaks = seq(-0.5, 1, 0.5),
                     limits = c(-0.65, 1)) +
  labs(x="Effect Size", y="Coefficients") +
  theme_bw() +
  theme(axis.text.x=element_text(size = 9, hjust = 1),
        axis.title=element_text(size = 16)) +
  panel_border(remove = TRUE)
#ggsave("outputs/figs/SF_06.png", width=12, height=6, dpi=300)


## END
