# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
## EPIDEMIC THRESHOLD SENSITIVITY ANALYSIS (T29 FORECAST) ======================

# Description:
#     Sensitivity analysis: T20 forecast for different epidemic thresholds,
#     including 50th percentile (median), 75th percentile and 95th percentile,
#     mean, mean + 1 standard deviation, mean + 2 standard deviations.

# Paper:
#     Compound and cascading effects of climatic extremes on dengue outbreak
#     risk in the Caribbean: an impact-based modelling framework with long-lag
#     and short-lag interactions

# Script authors:
#     Chloe Fletcher        ORCID: 0000-0002-6705-7605
#     Prof. Rachel Lowe     ORCID: 0000-0003-3939-7343

# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

## Source packages, functions and data -----------------------------------------

# load packages and functions
source("functions/00_packages_functions.R")

# read in harmonised data
data <- read.csv("data/prediction/barbados_fc_2024-03.csv")


## Set forecast model parameters -----------------------------------------------

vars <- c("tas.3mon.4", "spi6.5", "spi6.1")  # variables

fc_issue_year <- 2024  # calendar year forecast issued
fc_issue_month <- 3  # calendar month forecast issued
fc_issue_month_txt <- ifelse(fc_issue_month < 10, paste0("0", fc_issue_month),
                             as.character(fc_issue_month))  # add leading zero

fc_horizon <- 3
month <- ifelse(fc_horizon + fc_issue_month > 12,  # actual target month value
                (fc_horizon + fc_issue_month) %% 12,
                fc_horizon + fc_issue_month)

fname <- glue("outputs/prediction/pred_t20_{fc_issue_year}{fc_issue_month_txt}.rds")
pred <- readRDS(fname)  # read in pred file

# update dataframe
start_id <- match(month + 1, data$cal_month)
data <- data[start_id:nrow(data), ]

nyear <- length(unique(data$cal_year)) - 1
data$month <- rep(1:12, nyear, length.out = nrow(data))
data$year_index <- rep(1:nyear, each=12, length.out = nrow(data))

max_year_index <- max(data$year_index)

# set indices
m <- 12  # month index
s <- 1000  # number of samples


## Produce forecast visualisations ---------------------------------------------

# calculate moving monthly 50th percentile threshold
thresh_q50 <- data %>%
  filter(year_index < max_year_index) %>%  # excl. prediction year
  group_by(month) %>%
  summarise(q50 = quantile(cases, 0.5))

oprob_q50 <- round(sum(pred[,1] >= thresh_q50[m, "q50"][[1]]) * 100 / s, 3)

# calculate moving monthly 75th percentile threshold
thresh_q75 <- data %>%
  filter(year_index < max_year_index) %>%  # excl. prediction year
  group_by(month) %>%
  summarise(q75 = quantile(cases, 0.75))

oprob_q75 <- round(sum(pred[,1] >= thresh_q75[m, "q75"][[1]]) * 100 / s, 3)

# calculate moving monthly 95th percentile threshold
thresh_q95 <- data %>%
  filter(year_index < max_year_index) %>%  # excl. prediction year
  group_by(month) %>%
  summarise(q95 = quantile(cases, 0.95))

oprob_q95 <- round(sum(pred[,1] >= thresh_q95[m, "q95"][[1]]) * 100 / s, 3)

# calculate moving monthly mean threshold
thresh_m0s <- data %>%
  filter(year_index < max_year_index) %>%  # excl. prediction year
  group_by(month) %>%
  summarise(m0s = round(mean(cases, na.rm = TRUE), 2))

oprob_m0s <- round(sum(pred[,1] >= thresh_m0s[m, "m0s"][[1]]) * 100 / s, 3)

# calculate moving monthly mean + 1 standard deviation threshold
thresh_m1s <- data %>%
  filter(year_index < max_year_index) %>%  # excl. prediction year
  group_by(month) %>%
  summarise(m1s = round(mean(cases, na.rm = TRUE) + sd(cases, na.rm = TRUE), 2))

oprob_m1s <- round(sum(pred[,1] >= thresh_m1s[m, "m1s"][[1]]) * 100 / s, 3)

# calculate moving monthly mean + 1 standard deviation threshold
thresh_m2s <- data %>%
  filter(year_index < max_year_index) %>%  # excl. prediction year
  group_by(month) %>%
  summarise(m2s = round(mean(cases, na.rm = TRUE) + 2 * sd(cases, na.rm = TRUE), 2))

oprob_m2s <- round(sum(pred[,1] >= thresh_m2s[m, "m2s"][[1]]) * 100 / s, 3)

# font size
fs <- 18

# trigger thresholds
pred.i.q50 <- readRDS("supplementary/outputs/cross_validation/preds_i_50pct.rds")
pred.i.q75 <- readRDS("outputs/cross_validation/preds_i_adjustedyear_logdir.rds")
pred.i.q95 <- readRDS("supplementary/outputs/cross_validation/preds_i_95pct.rds")
pred.i.m0s <- readRDS("supplementary/outputs/cross_validation/preds_i_mean.rds")
pred.i.m1s <- readRDS("supplementary/outputs/cross_validation/preds_i_mean+1sd.rds")
pred.i.m2s <- readRDS("supplementary/outputs/cross_validation/preds_i_mean+2sd.rds")

i <- 4  # model number

trigger_q50 <- round(pred.i.q50$predictive_ability[i, "trigger"] * 100)
trigger_q75 <- round(pred.i.q75$predictive_ability[i, "trigger"] * 100)
trigger_q95 <- round(pred.i.q95$predictive_ability[i, "trigger"] * 100)
trigger_m0s <- round(pred.i.m0s$predictive_ability[i, "trigger"] * 100)
trigger_m1s <- round(pred.i.m1s$predictive_ability[i, "trigger"] * 100)
trigger_m2s <- round(pred.i.m2s$predictive_ability[i, "trigger"] * 100)

# plot gauges
g1_q50 <- gg.gauge.sa(oprob_q50, breaks = c(0, trigger_q50, 65, 80, 100)) +
  ggtitle("50th Percentile (Median)") +
  theme(plot.title = element_text(size = fs, hjust=0.5))

g1_q75 <- gg.gauge.sa(oprob_q75, breaks = c(0, trigger_q75, 50, 70, 100)) +
  ggtitle("75th Percentile") +
  theme(plot.title = element_text(size = fs, hjust=0.5))

g1_q95 <- gg.gauge.sa(oprob_q95, breaks = c(0, trigger_q95, 50, 75, 100)) +
  ggtitle("95th Percentile") +
  theme(plot.title = element_text(size = fs, hjust=0.5))

g1_m0s <- gg.gauge.sa(oprob_m0s, breaks = c(0, trigger_m0s, 50, 70, 100)) +
  ggtitle("Mean") +
  theme(plot.title = element_text(size = fs, hjust=0.5))

g1_m1s <- gg.gauge.sa(oprob_m1s, breaks = c(0, trigger_m1s, 50, 70, 100)) +
  ggtitle("Mean + 1 Standard Deviation") +
  theme(plot.title = element_text(size = fs, hjust=0.5))

g1_m2s <- gg.gauge.sa(oprob_m2s, breaks = c(0, trigger_m2s, 50, 70, 100)) +
  ggtitle("Mean + 2 Standard Deviation") +
  theme(plot.title = element_text(size = fs, hjust=0.5))

# plot gauges for 6 different thresholds
ggdraw() +
  draw_plot(g1_q50, x = 0, y = 2/3, width = 0.5, height = 1/3) +
  draw_plot(g1_m0s, x = 0.5, y = 2/3, width = 0.5, height = 1/3) +
  draw_plot(g1_q75, x = 0, y = 1/3, width = 0.5, height = 1/3) +
  draw_plot(g1_m1s, x = 0.5, y = 1/3, width = 0.5, height = 1/3) +
  draw_plot(g1_q95, x = 0, y = 0, width = 0.5, height = 1/3) +
  draw_plot(g1_m2s, x = 0.5, y = 0, width = 0.5, height = 1/3) +
  draw_plot_label(label = c("A", "B", "C", "D", "E", "F"),
                  size = 15,
                  x = c(0, 0.5, 0, 0.5, 0, 0.5),
                  y = c(1, 1, 2/3, 2/3, 1/3, 1/3)) +
  theme_bw(base_size = 14) +
  theme(panel.border = element_blank(), plot.title=element_text(hjust=0.5),
        panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
        legend.position="none", axis.text=element_blank(), axis.ticks=element_blank())
#ggsave("outputs/figs/SF_10.png", width=15, height=12, dpi=300)


## END