# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
## DROUGHT INDEX SENSITIVITY ANALYSIS ==========================================

# Description:
#     Compare the use of SPI with SPEI for modelling dengue risk, exploring
#     correlations, goodness of fit from univariable analysis, and replacing
#     SPI with SPEI for cross validation on selected candidate models.

# Paper:
#     Compound and cascading effects of climatic extremes on dengue outbreak
#     risk in the Caribbean: an impact-based modelling framework with long-lag
#     and short-lag interactions

# Script authors:
#     Chloe Fletcher        ORCID: 0000-0002-6705-7605
#     Prof. Rachel Lowe     ORCID: 0000-0003-3939-7343

# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

## Source packages, functions and data -----------------------------------------

# load packages and functions
source("functions/00_packages_functions.R")

# read in harmonised data
data <- read.csv("data/barbados_data.csv")

# read in spei variables
spei <- read.csv("data/barbados_spei_1998-2022.csv")


## Create lagged SPEI variables ------------------------------------------------

# number of columns in spei with temporal information
cols <- 3
start_year <- min(data$cal_year)

# create variable list
vars <- colnames(spei[(cols + 1):length(spei)])

# create lagged variables
for (v in vars){
  # create lags for each variable
  lags <- cbind(spei[, 1:cols], tsModel::Lag(spei[[v]], k = 0:nlag))
  lags <- lags[lags$cal_year >= start_year,]
  colnames(lags)[(cols + 1):length(lags)] <- paste(v, 0:nlag, sep=".")
  
  # merge lagged variables with spei
  spei <- merge(spei, lags, by = c("date", "cal_year", "cal_month"),
                all.x = TRUE)
  
  # remove original variable
  c <- grep(v, colnames(spei))[1]
  spei <- spei[, -c]
}

# remove .0 after 0-lag variables
colnames(spei) <- gsub(".0", "", colnames(spei))

# merge datasets
data <- merge(data, spei, by = c("date", "cal_year", "cal_month"), all.x = TRUE)

# correlation between spi and spei
cor(data[, c("spi1", "spi3", "spi6", "spi12", "spei1", "spei3", "spei6",
             "spei12")])[1:4, 5:8]

cor.test(data$spi1, data$spei1)
cor.test(data$spi3, data$spei3)
cor.test(data$spi6, data$spei6)
cor.test(data$spi12, data$spei12)


## Baseline models -------------------------------------------------------------

# read in intercept only model
int.mod <- readRDS("outputs/int_mod.rds")

# BASELINE MODEL 1
re1 <- paste("f(month, model = 'rw2', cyclic = TRUE, constr = TRUE,",
             "scale.model = TRUE, hyper = precision.prior)")
re2 <- "f(year_index, model = 'rw2', hyper = precision.prior)"
fe <- "logdir.4"
baseformula <- paste(c("cases ~ 1", re1, re2, fe), collapse = " + ")

# read in base model
base.mod <- readRDS("outputs/base_mod.rds")


## Create list of variables ----------------------------------------------------

# create list of variables for analysis
spi_vars <- grep("spi", colnames(data), value = TRUE)
spei_vars <- grep("spei", colnames(data), value = TRUE)


## Univariable analysis --------------------------------------------------------

# create dataframe of univariable models
uni.mod <- data.frame(vars = c(spi_vars, spei_vars))

# create list with gof dataframe, fitted values, fixed effects, random effects
uni.mod.out <- create.mod.out(uni.mod)

# run models and save ouputs
uni.mod.out <- fit.models(uni.mod.out, data, base.mod, int.mod, 
                          fname="univariable_mods")

# supplementary table 5
st_05 <- uni.mod.out$mod.gof[c(16:21, 44:49), c(1:2, 4, 6, 8)]


## Cross validation ------------------------------------------------------------

# set up original cross validation
source("functions/out_of_sample_progressive_adjusted.R")

spi6.mod.out <- readRDS("outputs/spi6_mods.rds")
candidate_mods <- c(23, 29, 30, 58, 59, 138, 173)

startyear <- data$cal_year[1]
startmonth <- data$cal_month[1]
startid <- nrow(data) - 12 * 10 + 1  # last 10 years of dataset

vars.p <- spi6.mod.out$mod.gof$vars[candidate_mods]
lags.p <- rep(3, length(vars.p))

re1 <- paste("f(month, model = 'rw2', cyclic = TRUE, constr = TRUE,",
             "scale.model = TRUE, hyper = precision.prior)")
re2 <- "f(year_index, model = 'rw2', hyper = precision.prior)"
fe <- "logdir.4"
res.p <- list(list(re1, re2, fe))

# function to convert spi to spei
convert_spi_to_spei <- function(vec, component = "long") {
  
  stopifnot("Invalid component" = component == "long" | component == "short")
  if (component == "long"){c <- 2} else {c <- 3} 
  
  sapply(vec, function(expr) {
    parts <- unlist(strsplit(expr, " \\* "))  # split string by " * "
    
    if (length(parts) >= c) {
      parts[c] <- sub("^spi", "spei", parts[c])  # spi to spei for component
    }
    
    paste(parts, collapse = " * ")  # recombine
  }, USE.NAMES = FALSE)  # output as character vector
}

# convert long-lag spi to spei
vars.pl <- convert_spi_to_spei(vars.p)
pred.pl <- out_of_sample(data = data, response = "cases", save = F,
                         vars = vars.pl, res = res.p, lags = lags.p,
                         start_id = startid, source_path = "functions",
                         file_path = "outputs")

# convert short-lag spi to spei
vars.ps <- convert_spi_to_spei(vars.p, "short")
pred.ps <- out_of_sample(data = data, response = "cases", save = F,
                         vars = vars.ps, res = res.p, lags = lags.p,
                         start_id = startid, source_path="functions",
                         file_path = "outputs")

# convert both lags spi to spei
vars.psl <- convert_spi_to_spei(convert_spi_to_spei(vars.p, "short"))
pred.psl <- out_of_sample(data = data, response = "cases", save = F,
                          vars = vars.psl, res = res.p, lags = lags.p,
                          start_id = startid, source_path="functions",
                          file_path = "outputs")


## END